#include "Image.h"

#include "Color.h"

#ifdef _WIN32
#include "Win32WicImage.h"
#endif

Image::Image(unsigned width, unsigned height, ImageData::Type dataType)
    :   mWidth(width),
        mHeight(height),
        mDataType(dataType)
{

}

Image::~Image()
{

}

std::unique_ptr<Image> Image::Create(unsigned width, unsigned height, ImageData::Type dataType)
{
    return std::make_unique<Image>(width, height, dataType);
}

void Image::initialize()
{
    if (mDataType == ImageData::Type::UCHAR || mDataType == ImageData::Type::UNKNOWN)
    {
        auto data = std::unique_ptr<ImageDataT<unsigned char> >();
        data->initialize(getBytesPerRow() * mHeight, 0);
        mData = std::move(data);
    }
    else
    {
        auto data = std::unique_ptr<ImageDataT<uint8_t> >();
        data->initialize(getBytesPerRow() * mHeight, 0);
        mData = std::move(data);
    }
}

void Image::setPixelValue(unsigned idx, unsigned jdx, const Color& color)
{
    if (!mData)
    {
        initialize();
    }

    const auto offset = jdx * getBytesPerRow() + idx * 3;
    mData->setDataItem(offset, color.getR());
    mData->setDataItem(offset + 1, color.getG());
    mData->setDataItem(offset + 2, color.getB());
}

unsigned Image::getBytesPerRow() const
{
    const auto bitsPerEntry = mBitDepth <= 8 ? 1 : 2;
    return mWidth * mNumChannels * bitsPerEntry;
}

unsigned Image::getWidth() const
{
    return mWidth;
}

unsigned Image::getHeight() const
{
    return mHeight;
}

unsigned Image::getBitDepth() const
{
    return mBitDepth;
}

PlatformImage* Image::getPlatformImage()
{
    if (!mPlatformImage)
    {
        mPlatformImage = std::make_unique<Win32WicImage>(this);
    }
    return mPlatformImage.get();
}

ImageData* Image::getData()
{
    return mData.get();
}

unsigned char Image::getAsUnsignedChar(unsigned idx, unsigned jdx) const
{
    return mData->getAsUnsignedChar(jdx * getBytesPerRow() + idx);
}

unsigned Image::getNumChannels() const
{
    return mNumChannels;
}

void Image::setWidth(unsigned width)
{
    mWidth = width;
}

void Image::setHeight(unsigned height)
{
    mHeight = height;
}

void Image::setBitDepth(unsigned bitDepth)
{
    mBitDepth = bitDepth;
}

void Image::setNumChannels(unsigned numChannels)
{
    mNumChannels = numChannels;
}

